/* LoginApp.m
   
   Login Window preferences module
	
   Created by Stefan Urbanek on 1999 Nov 1
*/

#import "LoginApp.h"
#import <AppKit/NSBox.h>
#import <AppKit/NSButtonCell.h>
#import <AppKit/NSMatrix.h>
#import <AppKit/NSPopUpButton.h>
#import <AppKit/NSTextField.h>
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSEnumerator.h>
#import <Foundation/NSDictionary.h>

#define DEFAULT_LOGINAPP_PATH @"/usr/GNUstep/Apps/Login.app"
#define LOGINAPP_DEFAULTS @"LoginAppPreferences"

static NSString *LoginThemeFileExtension = @"";
static NSString *NoThemesString = @"(no themes)";

enum
{
  WelcomeMessageSwitch = 1,
  HostnameSwitch,
  AnimationsSwitch,
  UserShutdownSwitch

};

@interface LoginApp(Private)
- (void)createView;
- (void)readThemes;
@end

@implementation LoginApp
- init
{
    [super init];
    [self createView];

    /* get Login.app path from:
           Preferences -> LoginAppPreferences -> LoginAppPath 
    */
    loginAppPath = [[[NSUserDefaults standardUserDefaults] 
                                   objectForKey:LOGINAPP_DEFAULTS]
                                   objectForKey:@"LoginAppPath"];
    [loginAppPath retain];

    if(loginAppPath == nil || [loginAppPath isEqualToString:@""])
    {
        loginAppPath = DEFAULT_LOGINAPP_PATH;
        NSLog(@"Using default path '%@'", loginAppPath);
    }

    [self revert:self];
    
    return self;
}
- (void)dealloc
{
	[loginAppPath release];
	[view release];
    [defaults release];
	[super dealloc];
}


- (void)readThemes
{
	NSDirectoryEnumerator *direnum;
	NSString *theme;
	NSString *themesPath = [loginAppPath stringByAppendingPathComponent:@"Themes"];
	NSString *type;

    [themes removeAllItems];

	direnum = [[NSDirectoryEnumerator alloc]
	                initWithDirectoryPath:themesPath
	                recurseIntoSubdirectories:NO
	                followSymlinks:NO
                    prefixFiles:NO];

	while (theme = [direnum nextObject]) { 
		type = [[direnum fileAttributes] objectForKey:NSFileType];
		if ([type isEqual:NSFileTypeDirectory]) { 
			[themes addItemWithTitle:theme];
		} 
	}
    
    if([themes numberOfItems] == 0)
    {
        [themes addItemWithTitle:NoThemesString];
    }
}

- (void)reloadDefaults
{
	NSString *preferencesFileName;
    preferencesFileName = [loginAppPath stringByAppendingPathComponent:@"Preferences"];

    [self readThemes];

	[defaults release];
	defaults = [NSMutableDictionary dictionaryWithContentsOfFile:preferencesFileName];
	[defaults retain];
}

- (void)revert:sender
{
	BOOL flag;

	[self reloadDefaults];
	
	[themes selectItemWithTitle:[defaults objectForKey:@"Theme"]];

	[welcomeMessage setStringValue:[defaults objectForKey:@"WelcomeMessage"]];	
	[restartMessage setStringValue:[defaults objectForKey:@"RestartMessage"]];	
	[powerMessage setStringValue:[defaults objectForKey:@"PowerOffMessage"]];	

	/* FIXME: use setState:flag */

	flag = [[[defaults objectForKey:@"DisableWelcomeMessage"] lowercaseString] isEqual:@"yes"];
	if(flag)
		[[switches cellWithTag:WelcomeMessageSwitch] setState:NSOnState];
	else
		[[switches cellWithTag:WelcomeMessageSwitch] setState:NSOffState];

	[welcomeMessage setEnabled:flag];
	
	flag = [[[defaults objectForKey:@"DisableUserShutdown"] lowercaseString] isEqual:@"yes"];
	if(flag)
		[[switches cellWithTag:UserShutdownSwitch] setState:NSOnState];
	else
		[[switches cellWithTag:UserShutdownSwitch] setState:NSOffState];

	flag = [[[defaults objectForKey:@"DisableHostname"] lowercaseString] isEqual:@"yes"];
	if(flag)
		[[switches cellWithTag:HostnameSwitch] setState:NSOnState];
	else
		[[switches cellWithTag:HostnameSwitch] setState:NSOffState];

	flag = [[[defaults objectForKey:@"DisableAnimations"] lowercaseString] isEqual:@"yes"];
	if(flag)
		[[switches cellWithTag:AnimationsSwitch] setState:NSOnState];
	else
		[[switches cellWithTag:AnimationsSwitch] setState:NSOffState];
}

- (void)save:sender
{
	NSString *preferencesFileName;
    preferencesFileName = [loginAppPath stringByAppendingPathComponent:@"Preferences"];

	[defaults setObject: [welcomeMessage stringValue] forKey:@"WelcomeMessage"];
	[defaults setObject: [restartMessage stringValue] forKey:@"RestartMessage"];
	[defaults setObject: [powerMessage stringValue] forKey:@"PowerOffMessage"];

	[defaults setObject: [themes titleOfSelectedItem] forKey:@"Theme"];

	/* FIXME: rewrite */
	if([[switches cellWithTag:WelcomeMessageSwitch] intValue])
		[defaults setObject: @"Yes" forKey:@"DisableWelcomeMessage"];
	else
		[defaults setObject: @"No" forKey:@"DisableWelcomeMessage"];

	if([[switches cellWithTag:HostnameSwitch] intValue])
		[defaults setObject: @"Yes" forKey:@"DisableHostname"];
	else	
		[defaults setObject: @"No" forKey:@"DisableHostname"];

	if([[switches cellWithTag:AnimationsSwitch] intValue])
		[defaults setObject: @"Yes" forKey:@"DisableUserShutdown"];
	else	
		[defaults setObject: @"No" forKey:@"DisableUserShutdown"];

	if([[switches cellWithTag:UserShutdownSwitch] intValue])
		[defaults setObject: @"Yes" forKey:@"DisableAnimations"];
	else	
		[defaults setObject: @"No" forKey:@"DisableAnimations"];

    /* here we have allways permission to write to this file,
       because LoginApp preferences are loaded only when Preferences.app
       was run as root.
    */
	[[defaults description] writeToFile:preferencesFileName atomically:YES];
}

- (void)switchesChanged:sender
{
	NSCell *cell = [sender selectedCell];
	if([cell tag] == WelcomeMessageSwitch)
	{
		[welcomeMessage setEnabled:[cell state]];
        [welcomeMessage setNeedsDisplay:YES];
//		[view setNeedsDisplay:YES];
	}
}

- (void)createView
{
	NSRect viewRect   = PreferencesLayoutViewFrame;
	NSRect buttonRect =    {{  4.0, 10.0},{168.0, 20.0}};
	NSRect themesBoxRect = {{ 10.0,165.0},{188.0, 64.0}};

	NSRect welcomeTRect  = {{ 16.0, 85.0},{110.0, 24.0}};
	NSRect welcomeFRect  = {{130.0, 85.0},{360.0, 24.0}};
	NSRect restartTRect  = {{ 16.0, 55.0},{110.0, 24.0}};
	NSRect restartFRect  = {{130.0, 55.0},{360.0, 24.0}};
	NSRect powerTRect    = {{ 16.0, 25.0},{110.0, 24.0}};
	NSRect powerFRect    = {{130.0, 25.0},{360.0, 24.0}};

	NSRect swRect        = {{320.0,140.0},{292.0,88.0}}; 
	
	NSTextField  *label;
	NSBox        *box;
	NSButtonCell *cellPrototype;
		
	view = [[NSView alloc] initWithFrame:viewRect];

	/* -- Themes -- */

	themes = [[NSPopUpButton alloc] initWithFrame:buttonRect];
    [themes addItemWithTitle:NoThemesString];

	box = [[[NSBox alloc] initWithFrame:themesBoxRect] autorelease];
	[box setTitle:@"Login Window Theme"];
	[box setBorderType: NSGrooveBorder];
	[box addSubview:themes];
	[view addSubview:box];

	/* -- Messages -- */
	welcomeMessage = [[[NSTextField alloc] initWithFrame:welcomeFRect] autorelease];
	restartMessage = [[[NSTextField alloc] initWithFrame:restartFRect] autorelease];
	powerMessage   = [[[NSTextField alloc] initWithFrame:powerFRect] autorelease];
	
	[view addSubview:welcomeMessage];
	[view addSubview:restartMessage];
	[view addSubview:powerMessage];

	label = [[[NSTextField alloc] initWithFrame:welcomeTRect] autorelease];
	[label setBezeled:NO];
	[label setEditable:NO];
	[label setBordered:NO];
	[label setStringValue:@"Welcome message:"];
	[label setDrawsBackground:NO];
	[label setSelectable:NO];
	[label setAlignment:NSRightTextAlignment];
	[view addSubview:label];

	label = [[[NSTextField alloc] initWithFrame:restartTRect] autorelease];
	[label setBezeled:NO];
	[label setEditable:NO];
	[label setBordered:NO];
	[label setStringValue:@"Restart message:"];
	[label setDrawsBackground:NO];
	[label setSelectable:NO];
	[label setAlignment:NSRightTextAlignment];
	[view addSubview:label];
	
	label = [[[NSTextField alloc] initWithFrame:powerTRect] autorelease];
	[label setBezeled:NO];
	[label setEditable:NO];
	[label setBordered:NO];
	[label setStringValue:@"Power Off message:"];
	[label setDrawsBackground:NO];
	[label setSelectable:NO];
	[label setAlignment:NSRightTextAlignment];
	[view addSubview:label];

	/* -- Switches -- */
	switches = [NSMatrix alloc];
	
	cellPrototype = [[[NSButtonCell alloc] init] autorelease] ;
	[cellPrototype setTarget:self];
	[cellPrototype setAction:@selector(switchesChanged:)];
	[cellPrototype setButtonType:NSSwitchButton];
	[cellPrototype setBordered:NO];
	
	[switches initWithFrame:swRect
			mode:NSHighlightModeMatrix
			prototype:cellPrototype
			numberOfRows:4
			numberOfColumns:1];

	[[switches cellAtRow:0 column:0] setTitle: @"Disable Welcome Message"];
	[[switches cellAtRow:0 column:0] setTag:WelcomeMessageSwitch];
	[[switches cellAtRow:1 column:0] setTitle: @"Disable Hostname"];
	[[switches cellAtRow:1 column:0] setTag:HostnameSwitch];
	[[switches cellAtRow:2 column:0] setTitle: @"Disable Animations"];
	[[switches cellAtRow:2 column:0] setTag:AnimationsSwitch];
	[[switches cellAtRow:3 column:0] setTitle: @"Disable User Shutdown"];
	[[switches cellAtRow:3 column:0] setTag:UserShutdownSwitch];

	[switches setTarget:self];
	[switches setAction:@selector(switcheChanged:)];

	[switches autorelease];
	[view addSubview:switches];
}
@end
